@@ -221,7 +221,7 @@ sign_in_identity (GoaKerberosProvider  *self,
 {
   GTask *operation_result;
 
-  operation_result = g_task_new (G_OBJECT (self), cancellable, callback, user_data);
+  operation_result = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (operation_result, (gpointer) identifier);
 
   g_object_set_data (G_OBJECT (operation_result),
@@ -822,7 +822,13 @@ on_initial_sign_in_done (GoaKerberosProvider *self,
 
   error = NULL;
   object_path = g_task_propagate_pointer (G_TASK (result), &error);
-  if (!g_task_had_error (G_TASK (result)) && remember_password)
+  if (error != NULL)
+    {
+      g_task_return_error (operation_result, error);
+      goto out;
+    }
+
+  if (remember_password)
     {
       GVariantBuilder  builder;
 
@@ -854,6 +860,7 @@ on_initial_sign_in_done (GoaKerberosProvider *self,
 
   g_task_return_boolean (operation_result, TRUE);
 
+ out:
   g_free (object_path);
   g_object_unref (operation_result);
 }
@@ -968,7 +975,7 @@ perform_initial_sign_in (GoaKerberosProvider *self,
 
   cancellable = g_cancellable_new ();
 
-  operation_result = g_task_new (G_OBJECT (self), cancellable, (GAsyncReadyCallback) on_account_signed_in, request);
+  operation_result = g_task_new (self, cancellable, (GAsyncReadyCallback) on_account_signed_in, request);
   g_task_set_source_tag (operation_result, object);
 
   g_object_set_data (G_OBJECT (operation_result),
